@@ -1,10 +1,12 @@
 import atexit
 import logging
-from typing import Any, Awaitable, Callable, Dict, List, Optional
+from typing import Any, Awaitable, Callable, Dict, List, Literal, Optional
 
 from motion.execute import Executor
 from motion.route import Route
-from motion.utils import DEFAULT_KEY_TTL, configureLogging, logger
+from motion.utils import DEFAULT_KEY_TTL, configureLogging
+
+logger = logging.getLogger(__name__)
 
 
 def is_logger_open(logger: logging.Logger) -> bool:
@@ -30,8 +32,10 @@ class ComponentInstance:
         serve_routes: Dict[str, Route],
         update_routes: Dict[str, List[Route]],
         logging_level: str = "WARNING",
-        disabled: bool = False,
+        update_task_type: Literal["thread", "process"] = "thread",
+        disable_update_task: bool = False,
         cache_ttl: int = DEFAULT_KEY_TTL,
+        redis_socket_timeout: int = 60,
     ):
         """Creates a new instance of a Motion component.
 
@@ -44,6 +48,9 @@ class ComponentInstance:
                 Logging level for the Motion logger. Uses the logging library.
                 Defaults to "WARNING".
         """
+        if update_task_type not in ["thread", "process"]:
+            raise ValueError("update_task must be either 'thread' or 'process'")
+
         self._component_name = component_name
         configureLogging(logging_level)
         # self._serverless = serverless
@@ -56,7 +63,7 @@ class ComponentInstance:
         self._cache_ttl = cache_ttl
 
         self.running = False
-        self.disabled = disabled
+        self.disable_update_task = disable_update_task
         self._executor = Executor(
             self._instance_name,
             cache_ttl=self._cache_ttl,
@@ -66,10 +73,21 @@ class ComponentInstance:
             load_state_func=load_state_func,
             serve_routes=serve_routes,
             update_routes=update_routes,
-            disabled=self.disabled,
+            update_task_type=update_task_type,
+            disable_update_task=self.disable_update_task,
+            redis_socket_timeout=redis_socket_timeout,
         )
         self.running = True
 
+    def __enter__(self) -> "ComponentInstance":
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback) -> None:  # type: ignore
+        self.shutdown()
+
+    def __del__(self) -> None:
+        self.shutdown()
+
     @property
     def instance_name(self) -> str:
         """Component name with a random phrase to represent
@@ -107,7 +125,7 @@ class ComponentInstance:
             c_instance.shutdown()
         ```
         """
-        if self.disabled:
+        if self.disable_update_task:
             return
 
         if not self.running:
@@ -116,7 +134,7 @@ class ComponentInstance:
         is_open = is_logger_open(logger)
 
         if is_open:
-            logger.info(f"Shutting down {self._instance_name}...")
+            logger.debug(f"Shutting down {self._instance_name}...")
 
         self._executor.shutdown(is_open=is_open)
 
@@ -146,7 +164,7 @@ class ComponentInstance:
         """
         return self._executor.version  # type: ignore
 
-    def write_state(self, state_update: Dict[str, Any]) -> None:
+    def write_state(self, state_update: Dict[str, Any], latest: bool = False) -> None:
         """Writes the state update to the component instance's state.
         If a update op is currently running, the state update will be
         applied after the update op is finished. Warning: this could
@@ -176,10 +194,17 @@ class ComponentInstance:
         Args:
             state_update (Dict[str, Any]): Dictionary of key-value pairs
                 to update the state with.
+            latest (bool, optional): Whether or not to apply the update
+                to the latest version of the state.
+                If true, Motion will redownload the latest version
+                of the state and apply the update to that version. You
+                only need to set this to true if you are updating an
+                instance you connected to a while ago and might be
+                outdated. Defaults to False.
         """
-        self._executor._updateState(state_update)
+        self._executor._updateState(state_update, force_update=latest)
 
-    def read_state(self, key: str) -> Any:
+    def read_state(self, key: str, default_value: Optional[Any] = None) -> Any:
         """Gets the current value for the key in the component instance's state.
 
         Usage:
@@ -205,11 +230,14 @@ class ComponentInstance:
 
         Args:
             key (str): Key in the state to get the value for.
+            default_value (Optional[Any], optional): Default value to return
+                if the key is not found. Defaults to None.
 
         Returns:
-            Any: Current value for the key.
+            Any: Current value for the key, or default_value if the key
+            is not found.
         """
-        return self._executor._loadState()[key]
+        return self._executor._loadState().get(key, default_value)
 
     def flush_update(self, dataflow_key: str) -> None:
         """Flushes the update queue corresponding to the dataflow
@@ -252,10 +280,11 @@ class ComponentInstance:
             dataflow_key (str): Key of the dataflow.
 
         Raises:
-            RuntimeError: If the component instance was initialized as disabled.
+            RuntimeError:
+                If the component instance was initialized as disable_update_task.
         """
-        if self.disabled:
-            raise RuntimeError("Cannot run a disabled component instance.")
+        if self.disable_update_task:
+            raise RuntimeError("Cannot run a disable_update_task component instance.")
 
         self._executor.flush_update(dataflow_key)
 
@@ -328,15 +357,14 @@ class ComponentInstance:
          Raises:
             ValueError: If more than one dataflow key-value pair is passed.
             RuntimeError:
-                If the component instance was initialized as disabled.
+                If flush_update is called and the component instance update
+                processes are disabled.
 
         Returns:
             Any: Result of the serve call. Might take a long time
             to run if `flush_update = True` and the update operation is
             computationally expensive.
         """
-        if self.disabled:
-            raise RuntimeError("Cannot run a disabled component instance.")
 
         serve_result = self._executor.run(
             key=dataflow_key,
@@ -357,8 +385,9 @@ class ComponentInstance:
         force_refresh: bool = False,
         flush_update: bool = False,
     ) -> Awaitable[Any]:
-        """Async version of run. Runs the dataflow (serve and update ops) for the
-        specified key.
+        """Async version of run. Runs the dataflow (serve and update ops) for
+        the specified key. You should use arun if either the serve or update op
+        is an async function.
 
         Example Usage:
         ```python
@@ -399,14 +428,12 @@ class ComponentInstance:
 
         Raises:
             ValueError: If more than one dataflow key-value pair is passed.
-            RuntimeError:
-                If the component instance was initialized as disabled.
+            If flush_update is called and the component instance update
+                processes are disabled.
 
         Returns:
             Awaitable[Any]: Awaitable Result of the serve call.
         """
-        if self.disabled:
-            raise RuntimeError("Cannot run a disabled component instance.")
 
         serve_result = await self._executor.arun(
             key=dataflow_key,
